/**
 @file		evb.c
 @brief 		functions to initialize EVB prepheral equipments
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "../mcu/types.h"
#include "../iinchip/socket.h"
#include "../util/util.h"
#include "../util/sockutil.h"
#include "w5100.h"
#include "wiznet.h"


#define	ATMEGA128_0WAIT		0
#define	ATMEGA128_1WAIT		1
#define	ATMEGA128_2WAIT		2
#define	ATMEGA128_3WAIT		3
#define 	ATMEGA128_NUM_WAIT	ATMEGA128_0WAIT


void InitWiznet(void) 
{
	cli();
		
#ifndef __DEF_IINCHIP_INT__	
	EICRA=0x00;
	EICRB=0x00;
	EIMSK=0x00;
	EIFR=0x00;
#else
	EICRA = 0x00;			// External Interrupt Control Register A clear
	EICRB = 0x02;			// External Interrupt Control Register B clear // edge 
	EIMSK = (1 << INT4);		// External Interrupt Mask Register : 0x10
	EIFR = 0xFF;			// External Interrupt Flag Register all clear
	DDRE &= ~(1 << INT4);		// Set PE Direction 
	PORTE |= (1 << INT4);		// Set PE Default value
#endif

 

#if (ATMEGA128_NUM_WAIT == ATMEGA128_0WAIT)
	MCUCR = 0x80;		
	XMCRA=0x40;
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_1WAIT)
	MCUCR = 0xc0;		// MCU control regiseter : enable external ram
	XMCRA=0x40;		// External Memory Control Register A : 
						// Low sector   : 0x1100 ~ 0x7FFF
						// Upper sector : 0x8000 ~ 0xFFFF
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_2WAIT )
	MCUCR = 0x80;
	XMCRA=0x42;
#elif ((ATMEGA128_NUM_WAIT == ATMEGA128_3WAIT)
	MCUCR = 0xc0;
	XMCRA=0x42;
#else
#error "unknown atmega128 number wait type"
#endif	


	#if (__DEF_IINCHIP_BUS__ == __DEF_IINCHIP_SPI_MODE__)
		//SPI Initialize
	   DDRB  |= 0x07;   	                     	// MISO=input, etc.=output
		                                             // PB3(MISO), PB2(MOSI), PB1(SCK), PB0(/SS)    
	   PORTB |= 0x01;                             // CS=1, waiting for SPI start
	   SPCR  = 0x50;                             // SPI mode 0, 4MHz
	   SPSR  = 0x01;                             // SPI2X=1
	#endif  
	
	sei();				// enable interrupts
}


