//=============================================================================
// Wiznet W5100 EVM Module Test Program
// by nexp76(nexp76@naver.com)
// [main.c]
//=============================================================================

#include <string.h>

//UART0 ϱ 
#define _UART0_ENABLE

#include "..\00_hal\system.h"
#include "..\01_drv\serial.c"
#define DebugPrint					U0_printf


#include "../01_drv/wiznet/mcu/types.h" 
#include "../01_drv/wiznet/util/config.h"
#include "../01_drv/wiznet/mcu/timer.h"
#include "../01_drv/wiznet/mcu/delay.h"
#include "../01_drv/wiznet/iinchip/w5100.h"
#include "../01_drv/wiznet/iinchip/socket.h"
#include "../01_drv/wiznet/util/sockutil.h"
#include "../01_drv/wiznet/iinchip/wiznet.h"
#include "../01_drv/wiznet/inet/httpd.h"
#include "rom/romfs.h"


#define LED_ON 1
   
//-----------------------------------------------------------------------------
#define MY_NET_MAC		"\x00\x08\xDC\x00\x00\x00"		// MY Mac Address : 00.08.DC.00.00.00
#define MY_NET_SIP			0xC0A80003	// MY Source IP     : 192.168.0.3
#define MY_NET_GWIP			0xC0A80001	// MY Gateway IP    : 192.168.0.1

#define MY_NET_SN		0xFFFFFF00	// MY Subnet Mask   : 255.255.255.0
#define MY_NET_DNS		0x00000000	// MY DNS Server IP : 0.0.0.0
#define MY_NET_MEMALLOC	0x55		// MY iinchip memory allocation

#define MY_CH_TYPE		LB_TCPS		// TCP Loop-Back Server
#define MY_LISTEN_PORT	5000		// MY Listen Port  : 5000 
#define MY_CONNECT_PORT	3000		// MY Connect Port : 3000
#define MY_SOURCE_PORT	3000		// MY Source Port  : 3000
#define MY_HTTP_PORT	80			// MY HTTP Port    : 80 
#define MY_CH_DESTIP		0xC0A80002	// MY Destination   : 192.168.0.2
//-----------------------------------------------------------------------------

#define TX_MAX_BUF_SIZE	512+256
#define RX_MAX_BUF_SIZE	256

#define SOCK_MYTEST			1
#define SOCK_TCPS			0
#define SOCK_HTTP			1

#define EVB_NET_SIP	"$SRC_IP_ADDRES$"
#define EVB_NET_GWIP	"$GW_IP_ADDRESS$"
#define EVB_NET_SN	"$SUB_NET__MASK$"
#define EVB_NET_DNS	"$DNS_SERVER_IP$"
#define EVB_NET_MAC	"$SRC_MAC_ADDRESS$"

#define EVB_LCD_TEXT 	"$LCD_TEXT_VALUE$"
#define EVB_LED0_IMG	"$LED0_IMG$"
#define EVB_LED1_IMG	"$LED1_IMG$"
#define EVB_LED0_STAT	"$LED_0$"
#define EVB_LED1_STAT	"$LED_1$"

extern void web_server(u_char ch);

static u_char* http_response;		/**< Pointer to HTTP response */
static st_http_request *http_request;	/**< Pointer to HTTP request */

unsigned char bchannel_start;;
extern NETCONF	NetConf;
extern CHCONF	ChConf;


//-----------------------------------------------------------------------------
void NetInit(void)
{
	u_char mac[6];
	unsigned long ip;
	memcpy(mac, MY_NET_MAC, 6);
	
	iinchip_init();

	ip = htonl(MY_NET_SIP);
	setSIPR((u_char*)&ip);

	ip = htonl(MY_NET_GWIP);
	setGAR((u_char*)&ip);

	ip = htonl(MY_NET_SN);
	setSUBR((u_char*)&ip);

	setSHAR(mac);
	
#ifdef __DEF_IINCHIP_INT__
	setIMR(0xEF);
#endif
	
	sysinit(MY_NET_MEMALLOC, MY_NET_MEMALLOC);
}
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Main Routine
//-----------------------------------------------------------------------------
int main(void)
{
	SystemInit();
	
	//Serial Init
	U0_Init(BAUD_115200);
	DebugPrint("Wiznet W5100 Test Program. - Web Server\r\n");


	//W5100 Chip Initialize
	//----------------------------
	InitWiznet();

	NetInit();
	//----------------------------
	
	#ifndef NO_USE_SOCKUTIL_FUNC
	//Display Net Configuration
	GetNetConfig();	
	#endif
	
	while (1)
	{
		//TCP Server Process
		web_server(SOCK_HTTP);
	}		
}
//-----------------------------------------------------------------------------


static u_int replace_sys_env_value(u_char* base,u_int len);	// Replace HTML's variables to system configuration value
static void proc_http(SOCKET ch, u_char *, int);		// processing HTTP

//processing http protocol , and excuting the followed fuction.
void web_server(u_char ch)
{
	int len;
	u_int wait_send=0;
	unsigned char rxdata_buf[RX_MAX_BUF_SIZE];
	
	http_request = (st_http_request*)rxdata_buf;		// struct of http request

	/* http service start */
	switch(getSn_SR(ch))
	{
	case SOCK_ESTABLISHED:
		if(bchannel_start==1)
		{
			DebugPrint("%d : Connected by %s(%u)\r\n",ch,inet_ntoa(GetDestAddr(ch)),GetDestPort(ch));
			bchannel_start = 2;
		}
		if ((len = getSn_RX_RSR(ch)) > 0)		
		{
			if ((u_int)len > MAX_URI_SIZE) len = MAX_URI_SIZE;				
			len = recv(ch, (u_char*)http_request, len);
			*(((u_char*)http_request)+len) = 0;
			
			DebugPrint("len=%d\r\n", len);
#ifdef WEB_DEBUG				
			DebugPrint( "- HTTP REQUEST -");
#endif				
			proc_http(ch, (u_char*)http_request, len);	// request is processed
		
			while(getSn_TX_FSR(ch)!= getIINCHIP_TxMAX(ch))
			{
				if(wait_send++ > 1500)
				{
#ifdef WEB_DEBUG				
					DebugPrint( "HTTP Response send fail");
#endif				
					break;
				}
				wait_1ms(1);
			}
			disconnect(ch);

			//DebugPrint( "- HTTP REQUEST -");
		}
		break;


	case SOCK_CLOSE_WAIT:   
#ifdef WEB_DEBUG	
		DebugPrint("CLOSE_WAIT : %d",ch);	// if a peer requests to close the current connection
#endif		
		disconnect(ch);
		bchannel_start = 0;
	case SOCK_CLOSED:                   
		if(!bchannel_start)
		{
			DebugPrint("[%d] : Web Server Started.",ch);
			bchannel_start = 1;
		}

		if(socket(ch, Sn_MR_TCP, DEFAULT_HTTP_PORT, 0x00) == 0)    /* reinitialize the socket */
		//DEFAULT_LISTEN_PORT
		{
			DebugPrint("\a%d : Fail to create socket.",ch);
			bchannel_start = 0;
		}
		else	listen(ch);
		break;
	}	// end of switch 
}



/**
 @brief	Analyse HTTP request and then services WEB.
*/
static void proc_http(SOCKET s,	u_char * buf, int length)
{
	prog_char * content;
	char* name;
	char * param;
	u_int file_len;
	u_int send_len;

	unsigned char rxdata_buf[RX_MAX_BUF_SIZE];	
	unsigned char txdata_buf[TX_MAX_BUF_SIZE];	
	
	http_response = (u_char*)txdata_buf;			// buffer for response header 
	http_request = (st_http_request*)rxdata_buf;	// struct of http request

	parse_http_request(http_request, buf);			// After analyze request, convert into http_request
	
	//method Analyze
	switch (http_request->METHOD)				
	{
	case METHOD_ERR :					
		send(s, (u_char *)ERROR_REQUEST_PAGE, strlen(ERROR_HTML_PAGE));
		break;
	case METHOD_HEAD:
	case METHOD_GET :
	case METHOD_POST :
		name = get_http_uri_name(http_request->URI);
		if (!strcmp(name, "/")) strcpy(name,"index.html");	// If URI is "/", respond by index.htm 
#ifdef WEB_DEBUG
		if(strlen(name)	< 80)	DebugPrint("PAGE : %s", name);
		else PRINTLN("TOO LONG FILENAME");
#endif 			
		find_http_uri_type(&http_request->TYPE, name);	//Check file type (HTML, TEXT, GIF, JPEG are included)
#ifdef WEB_DEBUG			
		PRINTLN("find_type() ok");
#endif		
		
		/* Search the specified file in stored binaray html image */
		if (!search_file(name, &content, &file_len))		//if the file do not exist then
		{
			send(s, (u_char *)ERROR_HTML_PAGE, strlen(ERROR_HTML_PAGE));	
#ifdef WEB_DEBUG				
			DebugPrint("Unknown Page");
#endif				
		} 
		else 						// if search file sucess 
		{
			
#ifdef WEB_DEBUG				
			DebugPrint("find file ok");
#endif				
			make_http_response_head(http_response, http_request->TYPE, (u_long)file_len);			
			send(s, http_response, strlen(http_response));

			while (file_len) 
			{
				if (file_len >= TX_MAX_BUF_SIZE-1)
					send_len = TX_MAX_BUF_SIZE-1;
				else	send_len = file_len;
				memcpy_P(http_response, content, send_len);
				*(http_response+send_len+1) = 0;
				
				// Replace htmls' system environment value to real value
				if(http_request->TYPE==PTYPE_HTML)
					send_len = replace_sys_env_value(http_response,send_len);
				send(s, http_response, send_len);
				content += send_len;
				file_len -= send_len;
			}
        	}
		break;
	default :
		break;
	}
}



/**
 @brief	Replace HTML's variables to system configuration value
*/
static u_int replace_sys_env_value(u_char* base, u_int len)
{
	u_char str[18];	
	u_char *ptr,*tptr;
	tptr = ptr = base;

	while((ptr=strchr(tptr,'$')))
	{
		if((tptr=strstr(ptr,EVB_NET_SIP)))
		{
			DebugPrint("REPLACE SIP");			
			tptr+=15;
		}
		else	// tptr == 0 && ptr!=0;
		{
			if(ptr==base)
			{
				DebugPrint("$ Charater");
				return len;
			}
			tptr = ptr;
			break;
		}
	}
	if(!ptr) return len;
	
	return (u_int)(tptr-base);
}
